// copyright 2002 Josh Levenberg, please see copying.txt

/* This is just a wrapper around gDisplay's caching interface.
    See display.h for details on how that works.
*/

#ifndef CACHE_HELPER_H
#define CACHE_HELPER_H
#include "display.h"
#include <assert.h>

namespace CacheHelper
  {
    extern bool gIsCaching;
    extern Coord *gCachedVerts;
    extern Index *gCachedIndex;
    extern Coord *gCurVert;
    extern Index *gCurIndex;

    // output a vertex
    inline void AllocateVertex(vec3_t v)
      {
        gCurVert[0] = v[0];
        gCurVert[1] = v[1];
        gCurVert[2] = v[2];
        gCurVert+=FloatsPerVert;
      }

    // output a vertex and return its index
    inline int AllocateVertexNum(vec3_t v)
      {
        gCurVert[0] = v[0];
        gCurVert[1] = v[1];
        gCurVert[2] = v[2];
        gCurVert+=FloatsPerVert;
        return (gCurVert-gCachedVerts)/FloatsPerVert-1;
      }

    // output the indices for the vertices of a triangle
    inline void PutTri(Index a, Index b, Index c)
      {
        gCurIndex[0] = a;
        gCurIndex[1] = b;
        gCurIndex[2] = c;
        gCurIndex+=3;
      }

    // start caching
    inline void BeginCache()
      {
        assert(!gIsCaching);
        gDisplay->BeginCache(gCachedVerts, gCachedIndex);
        gCurVert=gCachedVerts;
        gCurIndex=gCachedIndex;
        gIsCaching=true;
      }

    // done caching, return the cache identifier for the resulting cache
    inline Cache EndCache(int numtri)
      {
        assert(gIsCaching);
        assert(gCachedIndex+numtri*3==gCurIndex);
        gIsCaching=false;
        return gDisplay->FinishCache((gCurVert-gCachedVerts)/FloatsPerVert, numtri*3);
      }
  }
#endif
