// copyright 2002 Josh Levenberg, please see copying.txt
#include "uitype.h"

#ifdef MOVIE_OUTPUT
#include "dlod.h"
#include "model.h"
#include "screencap.h"

static double sAngle=0.0;
static double sTimeStep=0.01;
static double sGravity=1.5;
static double sTollerance=0.8;
static double sMaxVelocity=0.1;
static float sAvatarHdg = 0;
static float sAvatarVel[3] = {0, 0, 0};

static int sFramesToOutput=0;

void AnimateAvatar(double step)
  {
    if (!gPaused)
      {
        step *= sTimeStep;
        sAngle += step*2*PI/24;

        gAvatar[0] = 0.95*cos(sAngle);
        gAvatar[1] = 0.95*sin(sAngle);

        float height = gAvatar[2];
        float ground = gModel->LookUpHeight(gAvatar[0], gAvatar[1]);

        sAvatarHdg = sAngle + PI/2;

        double dv = step*sGravity;
        double dh = dv*step;
        float newheight = height+sAvatarVel[2]*step;

        if (newheight < ground + sTollerance*gAvatarHeight)
          { // way too low
            if (sAvatarVel[2]<0)
              sAvatarVel[2]=0;
            sAvatarVel[2] += dv;
            height = max(height+sAvatarVel[2]*step, 
                          ground + sTollerance*gAvatarHeight);
          }
        else
          {
            float idealheight = ground + gAvatarHeight;
            if (newheight > idealheight)
              if (newheight - dh > idealheight)
                sAvatarVel[2]-=dv; // falling
              else // following
                sAvatarVel[2]-=dv*(newheight-idealheight)/dh;
            else
              if (newheight + dh < idealheight)
                sAvatarVel[2] +=dv; // push up
              else // following
                sAvatarVel[2] +=dv*(idealheight-newheight)/dh;
            if (-sMaxVelocity>sAvatarVel[2])
              sAvatarVel[2]= -sMaxVelocity;
            else if (sMaxVelocity<sAvatarVel[2])
              sAvatarVel[2]= sMaxVelocity;
            height += sAvatarVel[2]*step;
          }

        gAvatar[2] = height;
      }
  }

static int sInstruction=0;
static double sInputFrameTime=1./30.;
static double sOutputFrameTime=.5/29.97;
static double sRemainderFrame=.5;
static double sTimeBeforeNextInstruction=-1.;

struct InstructType
  {
    double seconds;
    double framerate;
    const char *keys;
  };

static InstructType sInstructs[] =
  {
    { 45., 44., ">" },
    { 17., 44., "w" },
    { 17., 44., "ww2" },
    { 17., 24., "w1" },
    { 29., 10., "d" }
  };

static void GetNextInstruction()
  {
    if (sInstruction>=sizeof(sInstructs)/sizeof(sInstructs[0]))
      Quit();
    SimulateKeys(sInstructs[sInstruction].keys);
    gNumFrames=0;
    sTimeBeforeNextInstruction=sInstructs[sInstruction].seconds; 
    sInputFrameTime=1./sInstructs[sInstruction].framerate;
    SetCaptureCount(sInstruction ? sInstruction*1000+1000 : 0);
    ++sInstruction;
  }

static int sLastFramesOutput=0;

void glutIdle()
  { 
    if (sTimeBeforeNextInstruction<=-1.)
      sTimeBeforeNextInstruction=-.5;
    else if (sTimeBeforeNextInstruction<=0)
      GetNextInstruction();

    sRemainderFrame += sInputFrameTime/sOutputFrameTime;
    sFramesToOutput = int(sRemainderFrame); // next output frame time
    sRemainderFrame -= sFramesToOutput;
	// previous frame time
    double timethatwillelapse=sLastFramesOutput*sOutputFrameTime;
    sTimeBeforeNextInstruction -= timethatwillelapse;
    AnimateAvatar(timethatwillelapse/sInputFrameTime);
    sLastFramesOutput=sFramesToOutput;

    gCamera[0].mLocation[0] = gAvatar[0];
    gCamera[0].mLocation[1] = gAvatar[1];
    gCamera[0].mLocation[2] = gAvatar[2];

    gCamera[0].mLookAt[0] = gAvatar[0] + cos(sAvatarHdg)*0.8;
    gCamera[0].mLookAt[1] = gAvatar[1] + sin(sAvatarHdg)*0.8;
    gCamera[0].mLookAt[2] = gAvatar[2];

    gCamera[1].mLocation[0] = gCamera[0].mLookAt[0];
    gCamera[1].mLocation[1] = gCamera[0].mLookAt[1];

    gCamera[1].mLookAt[0] = gCamera[1].mLocation[0];
    gCamera[1].mLookAt[1] = gCamera[1].mLocation[1];
    gCamera[1].mLookAt[2] = gAvatar[2];

    glutPostRedisplay();
  }

void UIInit()
  {
  }

void UIKey(unsigned char key)
  {
    static double saved = 0.0;
    switch (key)
      {
        case '$':
          saved=sAngle;
          break;
        case '#':
          sAngle=saved;
          gPaused=true;
          break;
      }
  }

void UIPostDraw()
  {
    ScreenRGBCapture(sFramesToOutput);
    sFramesToOutput=0;
  }
#endif
