// copyright 2002 Josh Levenberg, please see copying.txt
#include "uitype.h"

#ifdef PROFILE_TEST_MODE
#include "dlod.h"
#include "model.h"

static double sOrbitRadius = 0.24;

static unsigned sFrameTarget=1;

static double sAngle=0.0;
static double sTimeStep=0.01;
static double sGravity=1.5;
static double sTollerance=0.8;
static float sAvatarHdg = 0;
static int sSpeedUp=1;

#define HARD_LIMIT (gAvatarHeight/2)

static void AnimateAvatar()
  {
    if (!gPaused)
      for (int count=0; count<sSpeedUp; ++count)
        {
          sAngle += sTimeStep*2*PI/24;

          gAvatar[0] = 0.5+sOrbitRadius*cos(sAngle);
          gAvatar[1] = 0.5+sOrbitRadius*sin(sAngle);

          float height = gAvatar[2];

          sAvatarHdg = sAngle + PI/2;

          double groundheight=gModel->LookUpHeight(gAvatar[0], gAvatar[1]);
          double idealheight=groundheight+gAvatarHeight;
          double minheight=groundheight+HARD_LIMIT;
          gAvatar[2] += (idealheight-gAvatar[2])*(0.1);
          if (gAvatar[2] < minheight)
            gAvatar[2]=minheight;
        }
  }

static int sInstruction=0;
static int sWarmUp=0;

static const char *sInstructs[] =
  {
    "",
    "d"
/*
    "<<<",      // Caching, Agg 1, Adaptive
    ">",        //              2
    ">",        //              3
    ">",        //              4
    ">",        //              5
    ">",        //              6
    ">",        //              7
    ">",        //              8
    "d",        // No caching, Agg 8, adaptive
    "<",        //                 7
    "<",        //                 6
    "<",        //                 5
    "<",        //                 4
    "<",        //                 3
    "<",        //                 2
    "<",        //                 1
    "d",        // Roam Standard
    "d",        // Full mesh, no caching
    "d",        // Full mesh, caching
*/
  };

static void GetNextInstruction()
  {
    if (sWarmUp)
      {
        ResetStats();
        sFrameTarget=2400;
        sWarmUp=0;
      }
    else
      {
        if (sInstruction)
          PrintStats();
        ResetStats();
        if (sInstruction>=sizeof(sInstructs)/sizeof(sInstructs[0]))
          Quit();
        SimulateKeys(sInstructs[sInstruction]);
        sFrameTarget=100; 
        sAngle=0;
        sWarmUp=1;
        ++sInstruction;
      }
  }

void glutIdle()
  { 
    if (gNumFrames>=sFrameTarget)
      GetNextInstruction();
    AnimateAvatar();

    gCamera[0].mLocation[0] = gAvatar[0];
    gCamera[0].mLocation[1] = gAvatar[1];
    gCamera[0].mLocation[2] = gAvatar[2];

    gCamera[0].mLookAt[0] = gAvatar[0] + cos(sAvatarHdg)*0.8;
    gCamera[0].mLookAt[1] = gAvatar[1] + sin(sAvatarHdg)*0.8;
    gCamera[0].mLookAt[2] = gAvatar[2];

    gCamera[1].mLocation[0] = gCamera[0].mLookAt[0];
    gCamera[1].mLocation[1] = gCamera[0].mLookAt[1];

    gCamera[1].mLookAt[0] = gCamera[1].mLocation[0];
    gCamera[1].mLookAt[1] = gCamera[1].mLocation[1];
    gCamera[1].mLookAt[2] = gAvatar[2];

    glutPostRedisplay();
  }

void UIInit()
  {
  }

void UIKey(unsigned char key)
  {
    static double saved = 0.0;
    switch (key)
      {
        case '\'':
          sSpeedUp *= 2;
          fprintf(gOutFile, "SpeedUp doubled to: %d\n", sSpeedUp);
          break;

        case ';':
          if (sSpeedUp>1)
            {
              sSpeedUp /= 2;
              fprintf(gOutFile, "SpeedUp halved to: %d\n", sSpeedUp);
            }
          break;

        case '$':
          saved=sAngle;
          break;
        case '#':
          sAngle=saved;
          gPaused=true;
          break;
      }
  }

void UIPostDraw()
  {
  }

#endif