// copyright 2002 Josh Levenberg, please see copying.txt
#ifndef VECTOR_3_H
#define VECTOR_3_H
#include <math.h>

const double PI=3.141592653589793238482843387;

typedef float vec_t;
typedef vec_t vec3_t[3];
struct vec3s_t
  {
    vec3_t m;
  };

// a = b - c
inline void vdiff(vec3_t a, const vec3_t b, const vec3_t c)
  {
    a[0] = b[0]-c[0];
    a[1] = b[1]-c[1];
    a[2] = b[2]-c[2];
  }

// a = b x c
inline void vcross(vec3_t a, const vec3_t b, const vec3_t c)
  {
    a[0] = b[1]*c[2]-b[2]*c[1];
    a[1] = b[2]*c[0]-b[0]*c[2];
    a[2] = b[0]*c[1]-b[1]*c[0];
  }

// a = a / |a|
inline void vnormalize(vec3_t vn)
  {
    // make normal have unit length
    vec_t vl = sqrt(vn[0]*vn[0]+vn[1]*vn[1]+vn[2]*vn[2]);
    vn[0] /= vl;
    vn[1] /= vl;
    vn[2] /= vl;
  }

// return a dot b
inline vec_t vdot(vec3_t a, vec3_t b)
  {
    return a[0]*b[0]+a[1]*b[1]+a[2]*b[2];
  }

// return (a-b) dot (a-b)
inline vec_t vdistsq(const vec3_t a, const vec3_t b)
  {
    vec_t x=a[0]-b[0], y=a[1]-b[1], z=a[2]-b[2];
    return x*x+y*y+z*z;
  }


#endif